from nonebot.plugin import on_command
from nonebot.adapters.onebot.v11 import Message, MessageEvent, GroupMessageEvent, MessageSegment
from nonebot.adapters.onebot.v11.helpers import HandleCancellation
from nonebot.params import State
from nonebot.typing import T_State

from .database import search_confirm, bgm_screenshoot1, bgm_screenshoot2



bgm = on_command(
    "bgm", priority=12
)



@bgm.got(
    "search_content", prompt='请输入要搜索的内容',
    parameterless=[HandleCancellation("已取消")]
)
@bgm.got(
    "search_input", prompt='请输入要搜索的类型编号: \n1.动画 2.现实人物 3.虚拟角色\n4.书籍 5.游戏 6.三次元 0.全部subject', 
    parameterless=[HandleCancellation("已取消")]
)
async def get_search_input(state: T_State = State()):
    search_input_base = str(state["search_input"])
    if search_input_base not in ["0","1","2","3","4","5","6"]:
        await bgm.reject('错误，请重新输入类型编号')
    search_input = int(search_input_base)
    search_content = str(state["search_content"]) 
    state["search_type"], state["search_menu"], state["search_sort"] = await search_confirm(search_input)
    search_sort, search_type = state["search_sort"], state["search_sort"]
    image, state["content"] = await bgm_screenshoot1(search_content,search_sort,search_type)
    if image == None or state["content"] == None:
        await bgm.finish(Message('没有搜索到相关内容哦~'))
    else:
        await bgm.send(MessageSegment.image(f"base64://{image}"), at_sender=True)
        


@bgm.got(
    "browse_num", prompt="请输入你想访问的条目的数字顺序：",
    parameterless=[HandleCancellation("已取消")]
)
async def get_browse_num(state: T_State = State()):
    browse_num_base = str(state["browse_num"])
    if browse_num_base not in ["0","1","2","3","4","5","6","7","8","9","10"]:
        await bgm.reject('错误，请重新输入类型编号')
    browse_num = int(browse_num_base)
    content, search_menu = state["content"], state["search_menu"]
    url, image_l, image_r = await bgm_screenshoot2(content,search_menu,browse_num)
    await bgm.finish(
            f"{url}"
            + MessageSegment.image(f"base64://{image_l}")
            + MessageSegment.image(f"base64://{image_r}"), at_sender=True
            )



from playwright.async_api import async_playwright
import re
import base64



async def get_browser():  
    p = await async_playwright().start()
    browser = await p.chromium.launch(headless=True)
    context = await browser.new_context()
    return context  

#判断搜索类型
async def search_confirm(search_input):    
    search_type_base = ['all',2,'prsn','crt',1,4,6]
    search_menu_base = [0,0,1,2,0,0,0]
    search_sort_base = ['subject_search','mono_search','mono_search']

    search_type = search_type_base[search_input]
    search_menu = search_menu_base[search_input]
    search_sort = search_sort_base[search_menu]
    return search_type, search_menu, search_sort



async def bgm_screenshoot1(search_content,search_sort,search_type):
    context = await get_browser()
    page = await context.new_page()
    # await page.goto(f"https://bgm.tv/{search_sort}/{search_content}?cat={search_type}")
    await page.goto(f"https://bgm.tv/{search_sort}/{search_content}?cat={search_type}",wait_until='commit')
    await page.reload()

    #处理网页版面
    content = await page.content()
    pattern = re.compile(r'<div id="columnSearchB"(.*?)<div id="columnSearchC"',re.S)
    content_base = re.search(pattern,content)
    if content_base == None:
        await page.close()
        return None, None

    else:
        content = content_base.group()

        #截图
        image = None
        image = await page.locator('#columnSearchB').screenshot()
        await page.close()
        return base64.b64encode(image).decode(), content



async def bgm_screenshoot2(content,search_menu,browse_num):   
    #处理进一步访问
    context = await get_browser()
    browse_type_base = ["subject","person","character"]
    browse_type = browse_type_base[search_menu]
    pattern_link = re.compile(f'<a href="/{browse_type}/(\d*)"')
    browse_link_base = re.findall(pattern_link,content)
    browse_link = ['ok']
    for i in browse_link_base:
        if browse_link.count(i) < 1:
            browse_link.append(i)
    browse_page = await context.new_page()
    await browse_page.goto(f"https://bgm.tv/{browse_type}/{browse_link[browse_num]}")
   
    #简介栏截图
    image_l = None
    image_l = await browse_page.locator('.infobox').screenshot()   
    
    #收藏盒图r的处理
    if search_menu == 0:
        browse_content= await browse_page.content()
        browse_r = browse_content.replace('<div class="SidePanel png_bg">','<div class="SidePanel png_bg" id="browse_r">')
        await browse_page.set_content(browse_r)

        #收藏盒截图
        image_r = None
        image_r = await browse_page.locator('#browse_r').screenshot()

    await browse_page.close()
    return f"https://bgm.tv/{browse_type}/{browse_link[browse_num]}", base64.b64encode(image_l).decode(), base64.b64encode(image_r).decode()  



from setuptools import setup
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(name="nonebot-plugin-bangumi-search",
    version="1.0.1",
    description="a nonebot2 plugin to help you search anime on bgm.tv and return some information to you",
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=["nonebot-plugin-bangumi-search"],
    author="Ankhyty",
    author_email="ankhyty@gmail.com",
    url="https://github.com/Ankhyty/nonebot-plugin-bangumi-search",
    python_requires=">=3.8.10"
)